#ifndef LATTICE_NEIGHBORHOOD_HH_
#define LATTICE_NEIGHBORHOOD_HH_


#include "NeighborVector.hh"

#include <map>

namespace ell
{
		/*! 
		 * A LatticeNeighborhood handles the
		 * the NeighborVector objects of a lattice and the 
		 * access to them.
		 */
	class LatticeNeighborhood
	{
	protected:

		const MoveAlphabet* moveAlph;	//!< the underlying move string alphabet
		NeighSet			neighSet;	//!< the NeighborVector data
		
			//! a crossreference from move index to NeighborVectors
		std::vector<const NeighborVector*>	neighVec;	

			//! a crossreference from vectors to NeighborVectors
		std::map<IntPoint,const NeighborVector*> vec2neigh;
		
	public:
		LatticeNeighborhood(	const MoveAlphabet* moveAlph_, 
								const NeighSet& neighbors);
								
		LatticeNeighborhood(	const LatticeNeighborhood& nh );
								
		virtual ~LatticeNeighborhood();
		
			//! Returns the number of elements of the neighborhood.
		virtual 
		unsigned int size() const;
		
			//! Returns whether or not a vector belongs to the neighborhood.
		virtual 
		bool isElement( const IntPoint& vector) const;
		
			//! Returns the corresponding NeighborVector.
			//! @param index has to be the index of an element of this
			//!				neighborhood (has to be less than the return value
			//!				of size())
		virtual 
		const NeighborVector& getElementByIndex(unsigned int index) const;

			//! Returns the corresponding NeighborVector.
			//! @param vector has to be element of this neighborhood
		virtual 
		const NeighborVector& getElement(const IntPoint& vector) const;
		
			//! Returns the corresponding NeighborVector.
			//! @param move has to be element of the underlying move string 
			//!				alphabet
		virtual 
		const NeighborVector& getElement(const Move& move) const;
		
			//! a constant iterator to access the elements of the neighborhood
		typedef NeighSet::const_iterator const_iterator;
		
			//! Returns a constant iterator to the first element of the 
			//! neighborhood
		virtual 
		const_iterator begin() const;
		
			//! Returns a constant iterator behind the last element of the 
			//! neighborhood
		virtual 
		const_iterator end() const;
	};

} // namespace ell

#include "LatticeNeighborhood.icc"

#endif /*LATTICE_NEIGHBORHOOD_HH_*/
