#include <ell/LatticeProteinPivotMoveStructCUBNormState.hh>
#include <ell/CLatticeProteinPivotMoveStructCUBNormState.hh>
#include <ell/assertell.hh>

namespace ell
{

	const LatticeDescriptorCUB
	 LatticeProteinPivotMoveStructCUBNormState::commonCUBLd;
	const LatticeModel
	 LatticeProteinPivotMoveStructCUBNormState::commonCUBLattice(
	  &commonCUBLd);
	const ContactEnergyFunction*
	 LatticeProteinPivotMoveStructCUBNormState::commonEnergy = NULL;	
	std::string LatticeProteinPivotMoveStructCUBNormState::commonSeqStr;
//////////////////////////////////////////////////////////
// LatticeProteinPivotMoveStructCUBNormState
//////////////////////////////////////////////////////////

	LatticeProteinPivotMoveStructCUBNormState::
	 LatticeProteinPivotMoveStructCUBNormState(
	  const std::string& moveString, const bool isAbsoluteMove)
	  : LatticeProteinPivotMoveState(&commonCUBLattice, commonEnergy,
		 commonSeqStr, moveString, isAbsoluteMove)
	 {}

	LatticeProteinPivotMoveStructCUBNormState::
	 LatticeProteinPivotMoveStructCUBNormState(
	  const LatticeProteinPivotMoveStructCUBNormState& latProtCUBNormState2)
	  : LatticeProteinPivotMoveState(latProtCUBNormState2)
	 {}

	LatticeProteinPivotMoveStructCUBNormState::
	 ~LatticeProteinPivotMoveStructCUBNormState() {
	}

	bool	
	LatticeProteinPivotMoveStructCUBNormState::
	 operator== (const State& state2) const {
		return normalize() == dynamic_cast<
			const LatticeProteinPivotMoveStructCUBNormState&>(state2).
			 normalize();
	}

	bool
	LatticeProteinPivotMoveStructCUBNormState::
	 operator!= (const State& state2) const {
		return normalize() != dynamic_cast<
			const LatticeProteinPivotMoveStructCUBNormState&>(state2).
			 normalize();
	}

	std::string
	LatticeProteinPivotMoveStructCUBNormState::normalize() const {
		std::string normalizedRelMove = latProt.getRelativeMoveString();

		assertell ( normalizedRelMove[0] == 'F',
		 "It's assumed that the relative move string starts with F.");

			// position of char which spans the 2nd and 3rd dimension
		size_t firstOf2ndDim, firstOf3rdDim;

		firstOf2ndDim = normalizedRelMove.find_first_not_of('F', 1);
			// 2nd spanned dimension is L/R and 3rd spanned dimension is U/D
		if (normalizedRelMove[firstOf2ndDim] == 'L'
			 || normalizedRelMove[firstOf2ndDim] == 'R') {
			firstOf3rdDim = normalizedRelMove.find_first_not_of("FLR", 1);
		}
			// 2nd spanned dimension is U/D and 3rd spanned dimension is L/R
		else {
			firstOf3rdDim = normalizedRelMove.find_first_not_of("FUD", 1);
		}

		switch (normalizedRelMove[firstOf2ndDim]) {
			case 'L' :
					// FLD...
				if (normalizedRelMove[firstOf3rdDim] == 'D') {
					break;
				}
					// FLU...
				else {
					swapCharsInString(normalizedRelMove,'L','U','R','D');
					break;
				}
			case 'R' :
					// FRD...
				if (normalizedRelMove[firstOf3rdDim] == 'D') {
					swapCharsInString(normalizedRelMove,'R','D','L','U');
					break;
				}
					// FRU...
				else {
					swapCharsInString(normalizedRelMove,'R','U','L','D');
					break;
				}
			case 'D' :
					// FDL...
				if (normalizedRelMove[firstOf3rdDim] == 'L') {
					swapCharsInString(normalizedRelMove,'D','L','U','R');
					break;
				}
					// FDR...
				else {
					swapCharsInString(normalizedRelMove,'U','L','D','R');
					break;
				}
			case 'U' :
					// FUL...
				if (normalizedRelMove[firstOf3rdDim] == 'L') {
					swapCharsInString(normalizedRelMove,'D','R','U','L');
					break;
				}
					// FUR...
				else {
					swapCharsInString(normalizedRelMove,'U','R','D','L');
					break;
				}
		}

		return normalizedRelMove;
	}

	void
	LatticeProteinPivotMoveStructCUBNormState::swapCharsInString (
	 std::string& s, char l, char d, char r, char u) const {
		for (size_t i = 1; i < s.size(); i++) {
			switch (s[i]) {
				case 'L':
					s[i] = l; break;
				case 'D':
					s[i] = d; break;
				case 'R':
					s[i] = r; break;
				case 'U':
					s[i] = u; break;
				default:
					break;
			}
		}
		return;
	}

	void
	LatticeProteinPivotMoveStructCUBNormState::setCommonContactEnergyFunction(
	 const ContactEnergyFunction* energy) {
		assertell ( commonEnergy == NULL,
		"It is not allowed to set the common energy function more than once.");
		commonEnergy = energy;
	}

	void
	LatticeProteinPivotMoveStructCUBNormState::setCommonSequenceString(
	 const std::string& seqStr) {
		assertell ( commonSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonSeqStr = seqStr;
	}

	LatticeProteinPivotMoveStructCUBNormState* 
	LatticeProteinPivotMoveStructCUBNormState::clone() const {
		return new LatticeProteinPivotMoveStructCUBNormState(*this);
	}

	CState*
	LatticeProteinPivotMoveStructCUBNormState::compressedClone() const {
		return new CLatticeProteinPivotMoveStructCUBNormState(
		 commonCUBLattice.parseMoveString(normalize()));
	}

	void
	LatticeProteinPivotMoveStructCUBNormState::compress(CState& cState) const {
		dynamic_cast<CLatticeProteinPivotMoveStructCUBNormState&>(cState)
		 = CLatticeProteinPivotMoveStructCUBNormState(
			commonCUBLattice.parseMoveString(normalize()));
	}

	State::NeighborListPtr 
	LatticeProteinPivotMoveStructCUBNormState::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	LatticeProteinPivotMoveStructCUBNormState::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	LatticeProteinPivotMoveStructCUBNormState::NeighborList::NeighborList(
	 			const LatticeProteinPivotMoveStructCUBNormState* _origin) 
	 : LatticeProteinPivotMoveState::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	LatticeProteinPivotMoveStructCUBNormState::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;

		LatticeProteinPivotMoveStructCUBNormState* latProtState = NULL;

		PivotMove firstMove = origin->firstPivotMove();

		if (firstMove != PivotMove(0,INVALID_INDEX)) {
			latProtState = new LatticeProteinPivotMoveStructCUBNormState(*origin);
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 firstMove.first, firstMove.second);
			myitstate->lastMove = firstMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(*itstate, latProtState);
			}
		}

		return latProtState;
	}

	State* 
	LatticeProteinPivotMoveStructCUBNormState::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

			// find next move
		PivotMove nextMove = origin->nextPivotMove(myitstate->lastMove);
			// no more moves possible
		if (nextMove == PivotMove(0,INVALID_INDEX)) {
			return NULL;
		}
		else {
			LatticeProteinPivotMoveStructCUBNormState* latProtState =
			 dynamic_cast<LatticeProteinPivotMoveStructCUBNormState*>(elem);
				// set latProtState to the state of origin of the neighbors
			latProtState->latProt = origin->latProt;
				// perform next move
			latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
			 nextMove.first, nextMove.second);
			myitstate->lastMove = nextMove;
				// if applied pivot move generates an invalid neighbor,
				// call next(..) to iterate neighbors until a VALID one is
				// found which will be returned
			if (!(latProtState->latProt.isValid())) {
				return next(itstate, latProtState);
			}
			return latProtState;
		}
	}

	// RandomNeighborList
	LatticeProteinPivotMoveStructCUBNormState::RandomNeighborList::RandomNeighborList(
	 		const LatticeProteinPivotMoveStructCUBNormState* _origin)
	 : LatticeProteinPivotMoveState::RandomNeighborList(_origin),
	   origin(_origin) 
	{
	}

	State* 
	LatticeProteinPivotMoveStructCUBNormState::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		LatticeProteinPivotMoveStructCUBNormState* latProtState = 
		 new LatticeProteinPivotMoveStructCUBNormState(*origin);

		return next(*itstate, latProtState);
	}

	State* 
	LatticeProteinPivotMoveStructCUBNormState::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		LatticeProteinPivotMoveStructCUBNormState* latProtState =
		 dynamic_cast<LatticeProteinPivotMoveStructCUBNormState*>(elem);

		PivotMove nextMove;

		if (myitstate->switchModeValue > 0) {
				// position and index of the next pivot move
			size_t pos, moveIndex;
			bool foundValidMove = false;

			while (!foundValidMove) {
					// move position has to be in [1, structure length)
				pos = (random() % (origin->latProt.getStructure().size()-1))+1;
					// moveIndex has to be in [0, number of available moves)
				moveIndex = random() % origin->pivotMoveSet.getMoveNumber();

				nextMove = PivotMove(pos, moveIndex);
					// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					myitstate->switchModeValue--;
					if (myitstate->switchModeValue == 0) {
						switchMode(myitstate);
						break;
					}
				}
					// move hasn't been chosen yet
				else {
						// set latProtState to the state of origin of the
						// neighbors
					latProtState->latProt = origin->latProt;
					latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
					 nextMove.first, nextMove.second);
						// if applied pivot move generates an invalid neighbor,
						// iterate neighbors until a VALID one is found which
						// will be returned
					foundValidMove = latProtState->latProt.isValid();
				}	
			}
		}
		if (myitstate->switchModeValue == 0) {
			size_t n = myitstate->unchosenMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {
				size_t i = random() % n;
					// set latProtState to the state of origin of the neighbors
				latProtState->latProt = origin->latProt;
				nextMove = myitstate->unchosenMoves[i];
				latProtState->pivotMoveSet.applyMoveInPlace(&latProtState->latProt,
				 nextMove.first, nextMove.second);
				myitstate->unchosenMoves[i] =
				 myitstate->unchosenMoves[n-1];
				myitstate->unchosenMoves.resize(n-1);
					// if applied pivot move generates an invalid neighbor,
					// call next(..) to iterate neighbors until a VALID one is
					// found which will be returned
				if (!(latProtState->latProt.isValid())) {
					return next(itstate, latProtState);
				}
			}
		}
		return latProtState;
	}

} // namespace ell
