#include <ell/RNAFreeEnergySingleMoveStructState.hh>
#include <ell/CRNAFreeEnergySingleMoveStructState.hh>
#include <ell/assertell.hh>

namespace ell
{

	std::string	RNAFreeEnergySingleMoveStructState::commonRnaSeqStr;
//////////////////////////////////////////////////////////
// RNAFreeEnergySingleMoveStructState
//////////////////////////////////////////////////////////

	RNAFreeEnergySingleMoveStructState::RNAFreeEnergySingleMoveStructState(
	  const std::string& rnaStructBracketNotStr)
	 : RNAFreeEnergySingleMoveState(commonRnaSeqStr, rnaStructBracketNotStr)
	{}

	RNAFreeEnergySingleMoveStructState::RNAFreeEnergySingleMoveStructState(
	 const RNAFreeEnergySingleMoveStructState& rnaFreeEnStruct)
	 : RNAFreeEnergySingleMoveState(rnaFreeEnStruct) 
	{}

	RNAFreeEnergySingleMoveStructState::~RNAFreeEnergySingleMoveStructState()	{
	}

	void
	RNAFreeEnergySingleMoveStructState::setCommonSequenceString(
	 const std::string& seqStr) {
		assertell ( commonRnaSeqStr.empty(),
		"It is not allowed to set the common sequence string more than once.");
		commonRnaSeqStr = seqStr;
	}

	RNAFreeEnergySingleMoveStructState* 
	RNAFreeEnergySingleMoveStructState::clone() const {
		return new RNAFreeEnergySingleMoveStructState(*this);
	}
	
	CState*
	RNAFreeEnergySingleMoveStructState::compressedClone() const {
		return new CRNAFreeEnergySingleMoveStructState(
		 rnaFreeEnergy.getStructure());
	}

	void
	RNAFreeEnergySingleMoveStructState::compress(CState& cState) const {
		dynamic_cast<CRNAFreeEnergySingleMoveStructState&>(cState)
		 = CRNAFreeEnergySingleMoveStructState(
			rnaFreeEnergy.getStructure());
	}

	State::NeighborListPtr 
	RNAFreeEnergySingleMoveStructState::getNeighborList() const {
		return NeighborListPtr(new NeighborList(this));
	}

	State::NeighborListPtr
	RNAFreeEnergySingleMoveStructState::getRandomNeighborList()
				const {
		return NeighborListPtr(new RandomNeighborList(this));
	}

	// NeighborList
	RNAFreeEnergySingleMoveStructState::NeighborList::NeighborList(
	 			const RNAFreeEnergySingleMoveStructState* _origin) 
	 : RNAFreeEnergySingleMoveState::NeighborList(_origin), origin(_origin)
	{
	}

	State* 
	RNAFreeEnergySingleMoveStructState::NeighborList::first(
		      State::NeighborList::ItState** itstate) const {
		if (*itstate != NULL) delete *itstate;

		ItState* myitstate = new ItState();
		*itstate = myitstate;
		
		RNAFreeEnergySingleMoveStructState* rnaState = NULL;

		SingleMove firstMove = origin->firstValidSingleMove();

		if (firstMove != SingleMove(0,RNAStructure::INVALID_INDEX)) {
			rnaState = new RNAFreeEnergySingleMoveStructState(*origin);
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(firstMove);
			myitstate->lastMove = firstMove;
		}
		return rnaState;
	}

	State* 
	RNAFreeEnergySingleMoveStructState::NeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const {
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);

		// find next move
		SingleMove nextMove = origin->nextValidSingleMove(myitstate->lastMove);
		// no more moves possible
		if (nextMove == SingleMove(0,RNAStructure::INVALID_INDEX)) {
			return NULL;
		}
		else {
			RNAFreeEnergySingleMoveStructState* rnaState =
			 dynamic_cast<RNAFreeEnergySingleMoveStructState*>(elem);
			// undo last move
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(myitstate->lastMove);
			// perform next move
			rnaState->rnaFreeEnergy.applySingleMoveInPlace(nextMove);
			myitstate->lastMove = nextMove;
			return rnaState;
		}
	}

	// RandomNeighborList
	RNAFreeEnergySingleMoveStructState::RandomNeighborList::RandomNeighborList(
	 		const RNAFreeEnergySingleMoveStructState* _origin)
	 : RNAFreeEnergySingleMoveState::RandomNeighborList(_origin),
	   origin(_origin)
	{
	}

	State* 
	RNAFreeEnergySingleMoveStructState::RandomNeighborList::first(
		      State::NeighborList::ItState** itstate) const 
	{
		if (*itstate != NULL) delete *itstate;

		*itstate = new ItState();
		RNAFreeEnergySingleMoveStructState* rnaState = 
		 new RNAFreeEnergySingleMoveStructState(*origin);

		return next(*itstate, rnaState);
	}

	State* 
	RNAFreeEnergySingleMoveStructState::RandomNeighborList::next(
		      State::NeighborList::ItState* itstate, State* elem) const 
	{
		assertell(elem != NULL, "Elem is not allowed to be NULL");

		ItState* myitstate = static_cast<ItState*>(itstate);
		RNAFreeEnergySingleMoveStructState* rnaState =
		 dynamic_cast<RNAFreeEnergySingleMoveStructState*>(elem);

		if (myitstate->switchModeValue > 0) {
			// positions the move will be applied to
			size_t pos1, pos2;
			SingleMove nextMove;
			bool foundValidMove = false;

			while (!foundValidMove) {
				pos1 = random() % origin->rnaFreeEnergy.getStructure().size();
				pos2 = random() % origin->rnaFreeEnergy.getStructure().size();
				if (pos1 == pos2) {
					continue;
				}
				nextMove = (pos1 < pos2) ? SingleMove(pos1, pos2) : SingleMove(pos2, pos1);
				// move has already been chosen
				if (!myitstate->chosenMoves.insert(nextMove).second) {
					myitstate->switchModeValue--;
					if (myitstate->switchModeValue == 0) {
						switchMode(myitstate);
						break;
					}
				}
				// move hasn't been chosen yet and is valid
				else if (origin->rnaFreeEnergy.
					 isValidSingleMove(nextMove)) {
						// set rnaState to the state of origin of the neighbors
					rnaState->rnaFreeEnergy = origin->rnaFreeEnergy;
					rnaState->rnaFreeEnergy.applySingleMoveInPlace(
					 nextMove);
					foundValidMove = true;
				}	
			}
		}
		if (myitstate->switchModeValue == 0) {
			size_t n = myitstate->unchosenValidMoves.size();
			if (n == 0) {
				return NULL;
			}
			else {
				size_t i = random() % n;
					// set rnaState to the state of origin of the neighbors
				rnaState->rnaFreeEnergy = origin->rnaFreeEnergy;
				rnaState->rnaFreeEnergy.applySingleMoveInPlace(
				 myitstate->unchosenValidMoves[i]);
				myitstate->unchosenValidMoves[i] =
				 myitstate->unchosenValidMoves[n-1];
				myitstate->unchosenValidMoves.resize(n-1);
			}
		}
		return rnaState;
	}

} // namespace ell
