

#include <iostream>
#include <ell/LatticeDescriptorCUB.hh>
#include <ell/LatticeProtein.hh>
#include <ell/PivotMoveSet.hh>


int main(int argc, char **argv) {

	using namespace ell;

	std::cout	<<"========== BEGIN OF TEST =================\n"
				<<" test class : LatticeProtein\n"
				<<"==========================================" <<std::endl;
				
		// erzeugen	
	std::cout	<<"=> creating ell::LatticeDescriptorCUB() = ld" <<std::endl;
		LatticeDescriptorCUB ld ;
	std::cout	<<"=> creating ell::LatticeModel(ld) = lm" <<std::endl;
		LatticeModel lm(&ld);
	std::cout	<<"=> creating ell::Alphabet(HP,1) = alph" <<std::endl;
		Alphabet alph("HP",1);
	std::cout	<<"=> creating ell::EnergyMatrix(0.0,2,2) = E" <<std::endl;
		EnergyMatrix E = EnergyMatrix(	0.0, 
									alph.getAlphabetSize(),
									alph.getAlphabetSize());
		E[alph.getIndex("H")][alph.getIndex("H")] = -1.0;
	std::cout	<<E <<std::endl;
	std::cout	<<"=> creating ell::ContactEnergyFunction( alph, E) = f" 
				<<std::endl;
		ContactEnergyFunction f(&alph, &E);
		std::string seqStr = "PHPHHPH";
	std::cout	<<"=> "<<seqStr <<" = seqStr" <<std::endl;
		std::string relMoveStr = "FRUULU";
	std::cout	<<"=> "<<relMoveStr <<"  = relMoveStr" <<std::endl;
		std::string absMoveStr = lm.getString(
									lm.relMovesToAbsMoves(
										lm.getDescriptor()->getSequence(
											relMoveStr)));
	std::cout	<<"=> "<<absMoveStr<<"  = absMoveStr" <<std::endl;

	std::cout	<<"=> creating ell::LatticeProtein(lm,f,seqStr,relMoveStr,false)"
				<<" = lp" 
				<<std::endl;
		LatticeProtein lp(&lm,&f,seqStr,relMoveStr,false);

		// getLatticeModel()
	std::cout	<<"->(*(lp.getLatticeModel()) == lm)  = "
				<<(*(lp.getLatticeModel()) == lm?"true":"false")
				<<std::endl;
				
		// getContactEnergyFunction()
	std::cout	<<"->(*(getContactEnergyFunction()) == f)  = "
				<<(*(lp.getContactEnergyFunction()) == f?"true":"false")
				<<std::endl;
				
		// getSequence()
	std::cout	<<"->(alph.getString(lp.getSequence()) == seqStr)  = "
				<<(alph.getString(lp.getSequence()) == seqStr ? "true":"false")
				<<std::endl;
				
		// getStructure()
	std::cout	<<"->(lm.getString(lp.getStructure()) == relMoveStr)  = "
				<<(lm.getString(lp.getStructure()) == relMoveStr ?
				   "true":"false")
				<<std::endl;

		// getStringRepresentation
	std::cout	<<"->getStringRepresentation()  = "
				<<lp.getStringRepresentation()
				<<std::endl;

		// getEnergy()
	std::cout	<<"->getEnergy()  = "
				<<lp.getEnergy()
				<<std::endl;

		// getAbsoluteMoveString()
	std::cout	<<"->getAbsoluteMoveString()  = "
				<<lp.getAbsoluteMoveString()
				<<std::endl;
	std::cout	<<"->(lp.getAbsoluteMoveString() == absMoveStr)  = "
				<<(lp.getAbsoluteMoveString() == absMoveStr?"true":"false")
				<<std::endl;
				
		// getAbsoluteMoves()
	std::cout	<<"->lm.getString(lp.getAbsoluteMoves()) == absMoveStr  = "
				<<( lm.getString(lp.getAbsoluteMoves()) == absMoveStr ? 
						"true":"false")
				<<std::endl;
				
		// getRelativeMoveString()
	std::cout	<<"->getRelativeMoveString()  = "
				<<lp.getRelativeMoveString()
				<<std::endl;
	std::cout	<<"->(lm.getString(lp.getRelativeMoves())"
				<<" == lp.getRelativeMoveString()  = "
				<<(lm.getString(lp.getRelativeMoves()) 
					== lp.getRelativeMoveString() ? "true":"false")
				<<std::endl;
				
		// getRelativeMoves()
	std::cout <<"->(lm.getString(lm.relMovesToAbsMoves(lp.getRelativeMoves()))"
				<<" == absMoveStr)  = "
				<<(lm.getString(lm.relMovesToAbsMoves(lp.getRelativeMoves())) ==
					absMoveStr ? "true":"false")
				<<std::endl;
				
		// get3Ddata()
	std::cout	<<"->get3Ddata().size()  = "
				<<lp.get3Ddata().size()
				<<std::endl;
				
		// getDRMSD()
	std::cout	<<"->getDRMSD(*this)  = "
				<<((int)(lp.getDRMSD(lp)*1000))
				<<std::endl;
				
		// isSelfAvoiding()
	std::cout	<<"->isSelfAvoiding()  = "
				<<(lp.isSelfAvoiding()?"true":"false")
				<<std::endl;

		// isConnected()
	std::cout	<<"->isConnected()  = "
				<<(lp.isConnected()?"true":"false")
				<<std::endl;

		// isValid()
	std::cout	<<"->isValid()  = "
				<<(lp.isValid()?"true":"false")
				<<std::endl;

	std::cout	<<"=> creating ell::PivotMoves(&lm) = pm" <<std::endl;
	PivotMoveSet pm(&lm);

		// getMoveNumber()
	std::cout	<<"->pm.getMoveNumber()  = "
				<<pm.getMoveNumber()
				<<std::endl;

		// applySingleMoveInPlace()
	std::cout	<<"->pm.applyMoveInPlace(&lp, 4, 1)" <<std::endl;
		pm.applyMoveInPlace(&lp, 4, 1);

		// getRelativeMoveString()
	std::cout	<<"->getRelativeMoveString()  = "
				<<lp.getRelativeMoveString()
				<<std::endl;

		// applySingleMove()
	std::cout	<<"->pm.applyMove(&lp, 4, 1).getRelativeMoveString()  = " <<std::endl;
		LatticeProtein* lp2 = pm.applyMove(&lp, 4, 1);
	std::cout	<<lp2->getRelativeMoveString()
				<<std::endl;
		delete lp2;

	std::cout	<<"============= END OF TEST ================\n" <<std::endl;
	
	return 0;
}
